/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (C) 2002   SEGGER Microcontroller Systeme GmbH               *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit.c   (for TOSHIBA H900H2)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.H"
#include "io900h2.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

#ifndef OS_FSYS
  #define OS_FSYS 20000000L      /* CPU clock frequency fc in [Hz]  */
#endif

#ifndef OS_PCLK                  /* Peripheral clock for Timer, SIO */
  #define OS_PCLK (OS_FSYS/4)    /* without influence of any        */
#endif                           /* prescaler                       */

#ifndef OS_TIMER
  #define OS_TIMER 8             /* Use 16 bit Timer8 per default   */
#endif
  
/* Select UART for embOSView.
   If you do not want (or can not due to hardware limitations) to dedicate
   a UART to embOSView, please define it to be -1
   Currently the standard code enables UART 0 per default
   and supports UART0 and UART1
*/

#ifndef OS_UART
  #define OS_UART 1
#endif    
#ifndef OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

#if OS_UART != (-1)
  #include "sic.h"               /* settings, definitions for UART  */
#endif

/*********************************************************************
*
*       OS_InitHW
*
**********************************************************************

Initialize the hardware required for the OS to run. This will work
on any target hardware, but may have to be tailored a little
(regarding the clock frequency). Of course the same holds true
if for some reason you choose to use an other timer.
*/

#if (OS_TIMER == 1) 
void OS_InitHW(void) {
  INTET01 = 0;                              /* Disable Timer 0 interrupt  */
  TRUN01  = 0;                              /* Stop Timer 0 & Timer 1     */
  TMOD01  = 0x41;                           /* 16bit mode, Prescaler 1    */
  TREG0   = (OS_PCLK/2/1000 & 0xFF);        /* low byte of time constant  */
  TREG1   = ((OS_PCLK/2/1000 >> 8) & 0xFF); /* high byte of time constant */
  INTET01 = 0x10;                           /* Enabe timer interrupt      */
  TRUN01  = 0x07;                           /* Start Timers & Prescaler   */
  OS_COM_Init();                            /* Initialize SIO (embOSView) */
}

#elif (OS_TIMER == 8)
void OS_InitHW(void) {
  TRUN8    = 0;                      /* Stop Timer 8                */
  INTET89  = 0x00;                   /* Disable Timer 8/9 interrupt */
  TFFCR8   = 0xC3;                   /* Disable Trigger             */
  TMOD8    = 0x01                    /* Select Prescale output      */
            |(1<<2)                  /* Enable clearing             */
            |(1<<5);                 /* Free capture trigger        */
  TREG9    = (OS_PCLK/2/1000);       /* load time constant          */
  INTET89 |= 0x10;                   /* Enabe timer interrupt       */
  TRUN8    = 0x07;                   /* Start Timers & Prescaler    */
  OS_COM_Init();                     /* Initialize SIO (embOSView)  */
}
#else
  #error "Selected Timer not supported by current embOS Version"
#endif

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
**********************************************************************

 Please note:
 This is basically the "core" of the idle loop
 This core loop can be changed, but:
 The idle loop does not have a stack of its own, therefore no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {      /* Idle loop: No task is ready to exec    */
  for (;;);               /* Nothing to do... wait for an interrupt */
}

/*********************************************************************
*
*       Run-time error reaction  (OS_Error)
*
**********************************************************************

 Run-time error reaction

 When this happens, a fatal error has occured and the kernel
 can not continue. In linux, the equivalent would be a
 "kernel panic"

 This routine can be modified to suit your needs ...
 E.g. a red LED could light up. When using an
 emulator, you may set a breakpoint here.
 In the release builds of the library (R), this routine is not required
 (as no checks are performed).
 In the stack check builds (S/SP), only error 120 may occur.
 In the debug builds(D/DP), all of the listed errors may occur.

  The following are the current errorcodes (V2.80) :
  (Please refer to the documentation for more info)
  
OS_ERR_STACK                          (120)
*** invalid or non-initialized data structures ***
OS_ERR_INV_TASK                       (128)
OS_ERR_INV_TIMER                      (129)
OS_ERR_INV_MAILBOX                    (130)
OS_ERR_INV_CSEMA                      (132)
OS_ERR_INV_RSEMA                      (133)

*** Using GetMail1 or PutMail1 or GetMailCond1 or PutMailCond1 on a non-1 byte mailbox  ***
OS_ERR_MAILBOX_NOT1                   (135)

***internal errors, please contact SEGGER Microcontrollersysteme ***
OS_ERR_MAILBOX_NOT_IN_LIST            (140)
OS_ERR_TASKLIST_CORRUPT               (142)


*** not matching routine calls or macro usage ***
OS_ERR_UNUSE_BEFORE_USE               (150)
OS_ERR_LEAVEREGION_BEFORE_ENTERREGION (151)
OS_ERR_LEAVEINT                       (152)
OS_ERR_DICNT                          (153)
OS_ERR_INTERRUPT_DISABLED             (154)

*** not a legal system call during interrupt ***
OS_ERR_ILLEGAL_IN_ISR                 (160)
*** not a legal system call during timer ***
OS_ERR_ILLEGAL_IN_TIMER               (161)

** double used data structures **
OS_ERR_2USE_TASK                      (170)
OS_ERR_2USE_TIMER                     (171)
OS_ERR_2USE_MAILBOX                   (172)
OS_ERR_2USE_BSEMA                     (173)
OS_ERR_2USE_CSEMA                     (174)
OS_ERR_2USE_RSEMA                     (175)

** Communication errors (OSCOM) **
OS_ERR_NESTED_RX_INT                  (180)
*/

void OS_Error(int ErrCode) {
  OS_EnterRegion();     /* Avoid further task switches */
  OS_DICnt =0;          /* Allow interrupts so we can communicate */
  OS_EI();
  OS_Status = ErrCode;
  while (OS_Status);
}

/*********************************************************************
*
*       Get time [cycles]
*
**********************************************************************

    This routine is required for task-info via embOS Viewer.
    It returns the system time in clock cycles.
    Can be used with 16 bit timers only, because other timers 
    can not be read back!
*/

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt;
  OS_U32 time = OS_Time;
  #if (OS_TIMER == 8) 
    TMOD8 |=  (1<<5);
    TMOD8 &= ~(1<<5);                /* Capture counter value       */
    t_cnt = CAP8;
    if (INTET89 & (1<<7)) {          /* If timer interrupt pending, */
      TMOD8 |=  (1<<5);
      TMOD8 &= ~(1<<5);              /* adjust  counter value       */
  	  t_cnt = CAP8;
      time++;
    }
    return (OS_PCLK/2/1000)*time  + t_cnt;
  #else
    return (OS_PCLK/2/1000)*time;
  #endif
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
**********************************************************************

Purpose:
   Convert Cycles into micro seconds. (For profiling only)
   If you have a clock frequency which is not a multiple of 1 MHz,
   you may have to modify this routine in order to get proper
   diagonstics.
Please note:
   This routine is required for profiling only.
   It does not affect operation of the OS.
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return (Cycles * 10) / (OS_PCLK/2/100000);
}

/*********************************************************************
*
*       OS_Tick interrupt
*
**********************************************************************
*/

void __interrupt OS_ISR_Tick (void) { OS_TickHandler(); }

/*********************************************************************
*
*       Communication for embOSView (UART 0)
*
**********************************************************************
*/

//#define BAUDDIVIDE ((OS_PCLK+OS_BAUDRATE*8L)/(OS_BAUDRATE*16L))
#define BAUDDIVIDE (OS_PCLK/OS_BAUDRATE/16)

#if (OS_UART == 0)
  #if ((BAUDDIVIDE > 16) || (BAUDDIVIDE == 0))
    #error "Unable to initialize requested Baudrate, Please modify code here..." 
  #endif

void OS_COM_Init(void) {
  INTES0   = 0;           /* Disable UART interrupts            */
  PFCR    &= ~(1<<1);     /* Set port to input (Enable RxD)     */
  PFCR    |=  (1<<0);     /* Set port to output (TxD push-pull) */
  PFFC    |=  (1<<0);     /* Select port function TxD0          */
  SC0MOD0  = 01           /* Use Baudrate generator             */
            |(1<<3)       /* 8 bit UART mode                    */
            |(1<<5);      /* Rx Enable                          */
  BR0CR    = (1<<4)       /* Prescaler set to 4 (T2)            */
            |BAUDDIVIDE;  /* Set calculated baudrate            */
  SC0CR    = 0;           /* Parity disable, Error reset        */
  INTES0   = 0x11;        /* Enable Rx- Tx- interrupts          */
}

void OS_COM_Send1(OS_U8 c) {
  SC0BUF = c;
}

/* The interrupt routines */
void __interrupt OS_ISR_Tx (void) {
  OS_EnterNestableInterrupt();  /*  We will enable interrupts   */ 
  OS_OnTx();
  OS_LeaveNestableInterrupt();
}

void __interrupt OS_ISR_Rx (void) {
  OS_U8 Data;
  OS_U8 error;
  OS_EnterNestableInterrupt();  /*  We will enable interrupts   */
  Data  = SC0BUF;
  if ((SC0CR & (OERR | PERR | FERR)) == 0) { /* check & reset errors */
    OS_OnRx(Data);                       
  }
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView (UART 1)
*
**********************************************************************
*/

#elif (OS_UART == 1)
  #if ((BAUDDIVIDE > 16) || (BAUDDIVIDE == 0))
    #error "Unable to initialize requested Baudrate, Please modify code here..."
  #endif

void OS_COM_Init(void) {
  INTES1   = 0;           /* Disable UART interrupts            */
  PFCR    &= ~(1<<4);     /* Set port to input (Enable RxD)     */
  PFCR    |=  (1<<3);     /* Set port to output (TxD push-pull) */
  PFFC    |=  (1<<3);     /* Select port function TxD0          */
  SC1MOD0  = 01           /* Use Baudrate generator             */
            |(1<<3)       /* 8 bit UART mode                    */
            |(1<<5);      /* Rx Enable                          */
  BR1CR    = BAUDDIVIDE;  /* Set calculated baudrate            */
  SC1CR    = 0;           /* Parity disable, Error reset        */
  INTES1   = 0x11;        /* Enable Rx- Tx- interrupts          */
}

void OS_COM_Send1(OS_U8 c) {
  SC1BUF = c;
}

/* The interrupt routines */
void __interrupt OS_ISR_Tx (void) {
  OS_EnterNestableInterrupt();  /*  We will enable interrupts   */ 
  OS_OnTx();
  OS_LeaveNestableInterrupt();
}

void __interrupt OS_ISR_Rx (void) {
  OS_U8 Data;
  OS_EnterNestableInterrupt();  /*  We will enable interrupts   */
  Data  = SC1BUF;
  if ((SC1CR & (OERR | PERR | FERR)) == 0) { /* reset errors    */
    OS_OnRx(Data);                       
  }
  OS_LeaveNestableInterrupt();
}

#elif (OS_UART == (-1))           /* UART for embOSView disabled */
  void __interrupt OS_ISR_Rx (void) {}
  void __interrupt OS_ISR_Tx (void) {}
  void OS_COM_Init(void) {}       
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* let OS know that transmitter is not busy  */
  }

#else  /* Selected UART not supported */
  #error "Selected UART NOT supported. Please add your own code !"
#endif /* (OS_UART==0) || (OS_UART==1) */

/*****  EOF  ********************************************************/

